// <copyright file="BusinessEntityToControlsLinker.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2010.10.13" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;
using System.Linq.Expressions;
using System.Windows.Forms;
using RO.Vema.Helper.BusinessObject;
using RO.Vema.Helper.BusinessObject.Services;

namespace RO.Vema.Helper.Client
{
    /// <summary>
    /// Clasa care creaza databinding si reguli de validare corespunzatoare.
    /// </summary>
    /// <typeparam name="TDataSource">tipul sursei de date cu care se face data binding si se valideaza datele</typeparam>
    public class BusinessEntityToControlsLinker<TDataSource>
    {
        /// <summary>
        /// type helper
        /// </summary>
        private readonly TypeHelper _typeHelper = new TypeHelper();

        /// <summary>
        /// the binding source
        /// </summary>
        private readonly BindingSource _winFormsBindingSource;

        /// <summary>
        /// Initializes a new instance of the <see cref="BusinessEntityToControlsLinker{TDataSource}"/> class.
        /// </summary>
        /// <param name="bindingSource">the binding source</param>
        /// <param name="errorProvider">the WinForms error provider</param>
        public BusinessEntityToControlsLinker(BindingSource bindingSource, ErrorProvider errorProvider)
        {
            _winFormsBindingSource = bindingSource;
        }

        /// <summary>
        /// Creates a DataBinding between a control property and a datasource property
        /// </summary>
        /// <typeparam name="TControl">The control type, must derive from System.Winforms.Control</typeparam>
        /// <param name="controlInstance">The control instance on wich the databinding will be added</param>
        /// <param name="controlPropertyAccessor">A lambda expression which specifies the control property to be databounded (something like textboxCtl => textboxCtl.Text)</param>
        /// <param name="datasourceMemberAccesor">A lambda expression which specifies the datasource property (something like datasource => datasource.Property) </param>
        /// <returns>the instance of the  binding created</returns>
        public Binding CreateLink<TControl>(
            TControl controlInstance, 
            Expression<Func<TControl, object>> controlPropertyAccessor, 
            Expression<Func<TDataSource, object>> datasourceMemberAccesor)
            where TControl : Control
        {
            return CreateLink(
                controlInstance,
                controlPropertyAccessor,
                datasourceMemberAccesor,
                DataSourceUpdateMode.OnPropertyChanged);
        }

        /// <summary>
        /// Creates a DataBinding between a control property and a datasource property
        /// </summary>
        /// <typeparam name="TControl">The control type, must derive from System.Winforms.Control</typeparam>
        /// <param name="controlInstance">The control instance on wich the databinding will be added</param>
        /// <param name="controlPropertyAccessor">A lambda expression which specifies the control property to be databounded (something like textboxCtl => textboxCtl.Text)</param>
        /// <param name="datasourceMemberAccesor">A lambda expression which specifies the datasource property (something like datasource => datasource.Property) </param>
        /// <param name="dataSourceUpdateMode">See control.DataBindings.Add method</param>
        /// <returns>the instance of the  binding created</returns>
        public Binding CreateLink<TControl>(
            TControl controlInstance, 
            Expression<Func<TControl, object>> controlPropertyAccessor, 
            Expression<Func<TDataSource, object>> datasourceMemberAccesor, 
            DataSourceUpdateMode dataSourceUpdateMode)
            where TControl : Control
        {
            string controlPropertyName = FlexReflector.GetPropertyName(controlPropertyAccessor);
            string sourcePropertyName = FlexReflector.GetPropertyName(datasourceMemberAccesor);

            return CreateBinding(controlInstance, controlPropertyName, sourcePropertyName, dataSourceUpdateMode);
        }

        /// <summary>
        /// Creates a DataBinding between a control property and a datasource property
        /// </summary>
        /// <typeparam name="TControl">The control type, must derive from System.Winforms.Control</typeparam>
        /// <param name="controlInstance">The control instance on wich the databinding will be added</param>
        /// <param name="controlPropertyName">A lambda expression which specifies the control property to be databounded (something like textboxCtl => textboxCtl.Text)</param>
        /// <param name="sourcePropertyName">A lambda expression which specifies the datasource property (something like datasource => datasource.Property) </param>
        /// <param name="dataSourceUpdateMode">See control.DataBindings.Add method </param>
        /// <returns>the instance of the  binding created</returns>
        private Binding CreateBinding<TControl>(
            TControl controlInstance, 
            string controlPropertyName, 
            string sourcePropertyName, 
            DataSourceUpdateMode dataSourceUpdateMode)
            where TControl : Control
        {
            Binding binding = controlInstance.DataBindings.Add(
                controlPropertyName, _winFormsBindingSource, sourcePropertyName, true, dataSourceUpdateMode);
            binding.Parse += OnBindingParse;
            return binding;
        }

        /// <summary>
        /// Event handler pentru <see cref="Binding.Parse"/>.
        /// </summary>
        /// <param name="sender">the sender</param>
        /// <param name="e">the event args</param>
        private void OnBindingParse(object sender, ConvertEventArgs e)
        {
            if (e.Value == null || e.Value.Equals(DBNull.Value))
            {
                object value;
                if (_typeHelper.GetDefaultValue(out value))
                {
                    e.Value = value;
                }
            }
        }
    }
}