// <copyright file="ControlHelper.cs" company="VEMA Calcul Electronic">
//   Copyright VEMA Calcul Electronic. All rights reserved.
// </copyright>
// <created>
//   <date value="2008.03.27" />
//   <author name="Emanuel Haisiuc" />
// </created>

using System;
using System.Drawing;
using System.Globalization;
using Infragistics.Win;
using Infragistics.Win.AppStyling;
using Infragistics.Win.AppStyling.Runtime;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;

namespace RO.Vema.Helper.Client
{
    public static class ControlHelper
    {
        private static readonly CultureInfo _romanianCultureInfo = new CultureInfo("ro-RO");
        private static Font _font;

        private static Font Font
        {
            get
            {
                if (_font == null)
                {
                    ApplicationStyleLibrary asl = new ApplicationStyleLibrary();
                    asl.LoadFromStyleManager();
                    StyleSetSettings ss = asl.StyleSets[asl.DefaultStyleSet];

                    StyleSettings role = ss.RoleStyles.GetStyle("Base", false);

                    _font = new Font(role.States[RoleState.Normal].Appearance.FontData.Name,
                                     role.States[RoleState.Normal].Appearance.FontData.SizeInPoints);
                }

                return _font;
            }
        }

        /// <summary>
        /// Redimensioneaza latimea drop down-ului combo-ului.
        /// </summary>
        /// <param name="combo">combo-ul pentru care se redimensiuneaza</param>
        public static void ResizeDropDownListWidthUltraComboEditor(UltraComboEditor combo)
        {
            Graphics ds = combo.CreateGraphics();
            int width = combo.Width;
            foreach (ValueListItem valueListItem in combo.Items)
            {
                width =
                    Math.Max(width,
                             (int)
                             decimal.Round(
                                 ((decimal)
                                  ds.MeasureString(valueListItem.DisplayText, combo.Font).
                                      Width + 60), 0));
            }
            combo.DropDownListWidth = width;
            ds.Dispose();
        }

        /// <summary>
        /// Seteaza latimea coloanei indicate in combo.
        /// </summary>
        /// <param name="combo">combo-ul pentru care se seteaza latimea</param>
        /// <param name="columnName">numele coloanei careia i se va modifica latimea</param>
        public static void SetComboColumnWidth(UltraCombo combo, string columnName)
        {
            Graphics ds = combo.CreateGraphics();
            int width = combo.Width;
            foreach (UltraGridRow row in combo.Rows)
            {
                int colWidth =
                    (int)
                    decimal.Round(((decimal) ds.MeasureString(row.Cells[columnName].Text, combo.Font).Width + 60), 0);
                width = Math.Max(width, colWidth);
            }

            ds.Dispose();
            combo.DisplayLayout.Bands[0].Columns[columnName].Width = width;
        }

        /// <summary>
        /// Seteaza latimea coloanei indicate in grid.
        /// </summary>
        /// <param name="grid">grid-ul pentru care se seteaza latimea</param>
        /// <param name="column"> </param>
        /// <param name="hideEmptyColumns"> </param>
        public static void SetGridColumnWidth(UltraGridBase grid, UltraGridColumn column, bool hideEmptyColumns)
        {
            Graphics ds = null;
            int width = 81;
            column.Width = width;

            if (grid.Rows.Count == 0)
            {
                return;
            }

            bool hasContent = (column.CellActivation == Activation.AllowEdit) || !column.IsBound;

            try
            {
                ds = grid.CreateGraphics();

                foreach (UltraGridRow row in grid.Rows)
                {
                    if (row.IsFilteredOut)
                    {
                        continue;
                    }

                    int colWidth =
                        (int) decimal.Round(((decimal) ds.MeasureString(row.GetCellText(column), Font).Width + 10), 0);
                    width = Math.Max(width, colWidth);

                    hasContent = hasContent || !string.IsNullOrEmpty(row.GetCellText(column));
                }
            }
            finally
            {
                if (ds != null)
                {
                    ds.Dispose();
                }
            }

            column.Width = width;

            if (hideEmptyColumns)
            {
                if (grid.Rows.Count > 0)
                {
                    column.Hidden = !hasContent;
                }
            }
        }

        /// <summary>
        /// Elimina caracterele de afisare ale sablonul de culegere din valoare, si intoarce sirul de caractere reprezentand numarul.
        /// </summary>
        /// <remarks>
        /// Daca dupa eliminarea caracterelor de afisare ale sablonului de culegere 
        /// se obtine separatorul zecimal sau minus si separatorul unitar atunci valoarea intoarsa va fi 0.
        /// </remarks>
        /// <param name="value">valoarea de pregatit pentru parsare</param>
        /// <returns>sirul de caractere reprezentand numarul de parsat</returns>
        public static decimal GetDecimal(string value)
        {
            return GetDecimal(value, '_');
        }

        /// <summary>
        /// Elimina caracterele de afisare ale sablonul de culegere din valoare, si intoarce sirul de caractere reprezentand numarul.
        /// </summary>
        /// <remarks>
        /// Daca dupa eliminarea caracterelor de afisare ale sablonului de culegere 
        /// se obtine separatorul zecimal sau minus si separatorul unitar atunci valoarea intoarsa va fi 0.
        /// </remarks>
        /// <param name="value">valoarea de pregatit pentru parsare</param>
        /// <param name="promptChar">valoarea caracterului mastii de culegere de date</param>
        /// <returns>sirul de caractere reprezentand numarul de convertit</returns>
        public static decimal GetDecimal(string value, char promptChar)
        {
            if (string.IsNullOrEmpty(value))
            {
                return decimal.Zero;
            }

            string newValue = value.Replace(new string(promptChar, 1), string.Empty);
            if (string.IsNullOrEmpty(newValue))
            {
                return decimal.Zero;
            }

            newValue =
                newValue.Replace(
                    CultureInfo.CurrentCulture.NumberFormat.CurrencyDecimalSeparator,
                    CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator);

            bool isDecimalPoint = (newValue == CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator);
            if (isDecimalPoint)
            {
                return decimal.Zero;
            }

            bool isMinusAndDecimalPoint =
                (newValue == string.Format("-{0}", CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator));
            if (isMinusAndDecimalPoint)
            {
                return decimal.Zero;
            }

            decimal decimalValue;
            if (!decimal.TryParse(newValue, out decimalValue))
            {
                decimalValue = decimal.Zero;
            }

            return decimalValue;
        }

        /// <summary>
        /// Elimina caracterele de afisare ale sablonul de culegere din valoare, si intoarce sirul de caractere reprezentand numarul.
        /// </summary>
        /// <remarks>
        /// Daca dupa eliminarea caracterelor de afisare ale sablonului de culegere 
        /// se obtine separatorul zecimal sau minus si separatorul unitar atunci valoarea intoarsa va fi 0.
        /// </remarks>
        /// <param name="value">valoarea de pregatit pentru parsare</param>
        /// <param name="promptChar">valoarea caracterului mastii de culegere de date</param>
        /// <returns>sirul de caractere reprezentand numarul de convertit</returns>
        public static int GetInteger(string value, char promptChar)
        {
            if (string.IsNullOrEmpty(value))
            {
                return 0;
            }

            string newValue = value.Replace(new string(promptChar, 1), string.Empty);
            if (string.IsNullOrEmpty(newValue))
            {
                return 0;
            }

            int intValue;
            if (!int.TryParse(newValue, NumberStyles.Integer, _romanianCultureInfo, out intValue))
            {
                intValue = 0;
            }

            return intValue;
        }
    }
}